/*
 * DE0 USB Host demo.
 * (HUB, Flash memory, mouse, keyboard, gamepad)
 */


#include <stdio.h>
#include <system.h>
#include <unistd.h>
#include <sys/alt_irq.h>
#include "io.h"
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_timer_regs.h"
#include "altera_avalon_timer.h"
#include "altera_avalon_timer.h"
#include "alt_types.h"
#include "usbh_hw_reg.h"
#include "usbh_env.h"
#include "video.h"
#include "pff.h"

#define INT_MODE

/* File object */
void die (		/* Stop with dying message */
	FRESULT rc	/* FatFs return value */
)
{
  switch(rc) {
  case FR_DISK_ERR:	/* 1 */
	  printf("Failed with FR_DISK_ERR.\n");
	  break;
  case FR_NOT_READY:		/* 2 */
	  printf("Failed with FR_NOT_READY.\n");
	  break;
  case FR_NO_FILE:			/* 3 */
	  //printf("Failed with FR_NO_FILE.\n");
	  printf("FONT.BMP not found in MSD.\n");
	  break;
  case FR_NO_PATH:			/* 4 */
	  printf("Failed with FR_NO_PATH.\n");
	  break;
  case FR_NOT_OPENED:		/* 5 */
	  printf("Failed with FR_NOT_OPENED.\n");
	  break;
  case FR_NOT_ENABLED:		/* 6 */
	  printf("Failed with FR_NOT_ENABLED.\n");
	  break;
  case FR_NO_FILESYSTEM:	/* 7 */
	  printf("Failed with FR_NO_FILESYSTEM.\n");
	  break;

  }

  for (;;) ;
}

st_hid_gamepad gamepad_data;
un_hid_mouse mouse_data;
un_hid_keyboard keyboard_data;
#define keyboard_buffer (*(volatile char *)(KEY_BUFFER)

volatile int video_status;
volatile int fb_front;

// keyboard call back
void on_keyboard(int key, int modifier) {
  int len;
  volatile char *kb;
  kb = (volatile char *)(KEY_BUFFER);
  len = strlen(kb);
  //printf("len= %d\n",len);
  if (key == 0x7f) {
    if (len >0) kb[len-1] = 0;
  } else {
	 if (len <12) {
     kb[len] = key;
	 kb[len+1] = 0;
	 }
  }
  if (key != 0) printf("key changed %c %x\r\n",key,modifier);
  keyboard_data.BYTE.modifier = modifier;
  keyboard_data.BYTE.keycode[0] = key;
}
// gamepad callback
void on_gamepad(int buttons, int x, int y) {
  static int initial_flag = 1;
  static int pb = 0, px = 0, py = 0;
  if (!initial_flag) {
    if ((pb != buttons)||(px != x)||(py!=y))
    printf("gamepad changed %x %x %x\r\n",buttons,x,y);
  }

  initial_flag =0;
  pb = buttons; px = x; py = y;
  gamepad_data.BYTE.x = x;
  gamepad_data.BYTE.y = y;
  gamepad_data.BYTE.b0.BYTE =buttons;
}
// mouse callback
void on_mouse(int buttons, int x, int y, int z) {
  mouse_data.BYTE.x = x;
  mouse_data.BYTE.y = y;
  mouse_data.BYTE.b = buttons;
  printf("mouse changed %x %x %x %x\r\n",buttons,x,y,z);
}

FATFS fatfs;		/* File system object */

static void handle_timer_interrupts(void* context);
void init_timer_interrupt(void);
unsigned int value;
unsigned int delta;

#ifdef INT_MODE
void init_timer_interrupt(void){
    //int x;
	int* ptr;
    IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_1_BASE, (1<<3) | (1 << 1) |(1 << 0));
    IOWR_ALTERA_AVALON_TIMER_STATUS(TIMER_1_BASE,0);// Clear TO Bit(Reaching 0)
    //x = IORD_ALTERA_AVALON_TIMER_PERIODL(TIMER_1_BASE);
    //printf("timer l %x\n",x);
    //x = IORD_ALTERA_AVALON_TIMER_PERIODH(TIMER_1_BASE);
    //printf("timer h %x\n",x);
    alt_ic_isr_register(TIMER_1_IRQ_INTERRUPT_CONTROLLER_ID, TIMER_1_IRQ,handle_timer_interrupts, ptr, 0x0); //Register Interrupt
    IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_1_BASE,(1<<2) | (1 << 1) | (1 << 0) );//Start Timer, IRQ enable, Continuous enable
}
static void handle_timer_interrupts(void* context){
  IOWR_ALTERA_AVALON_TIMER_STATUS(TIMER_1_BASE,0);//Clear TO(timeout) bit)
  IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_1_BASE, (1<<3) | (1 << 1) |(0 << 0));
  uh_update();
  IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_1_BASE,(1<<2) | (1 << 1) | (1 << 0) );//Start Timer, IRQ enable, Continuous enable
}
#endif

int main()
{
    volatile char *kb;
    int i,fcnt;

    printf("DE0 USB Host demo\n\r");
    value = 0;

    video_status = 0;
    kb = (volatile char *)(KEY_BUFFER);
    for (i=0;i<640;i++) kb[i] = 0;

    uh_init();
    uh_mouse_attach_func(on_mouse);
    uh_keyboard_attach_func(on_keyboard);
    uh_gamepad_attach_func(on_gamepad);
    video_init();
    if (uh_msd_available(0)) {
      get_bitmap_data("FONT.BMP");
    } else printf("MSD not found. FONT.BMP not loaded\n");

    // HID loop
    while(1) {
  	  video_update();
#ifdef INT_MODE
  	  fcnt = video_swap();
  	  if (fcnt == 3 ) init_timer_interrupt();  // register timer interrupt
                                               // after finishing USB flash access
#else
  	  uh_update();
  	  video_swap();
#endif
    }
    return 0;
}
