//=======================================================================
//                         DE0 USBHOST
//
// File:
//   fm_top.v
//
// Abstract:
//   OpenCores usbhost wrapper
//
//  Created:
//    27 March 2012
//======================================================================
//
// Copyright (c) 2013, Kenji Ishimaru
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
//  -Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//  -Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Revision History
// $Date: 2013-09-18 19:15:35 +0900 (Wed, 18 Sep 2013) $
// $Rev: 4 $

module fm_usb_wrapper (
  // AVALON Slave Interface
  input           clk_core,
  input           rst_x,
  input  [5:0]    i_av_adr,
  input  [3:0]    i_av_be,
  input           i_av_r,
  output [31:0]   o_av_rd,
  input           i_av_w,
  input  [31:0]   i_av_wd,
  output          o_av_wait,
  // USB int
  output          o_int,
  // USB Host
  input           clk48m,
  inout           usb_dp,
  inout           usb_dn
);
  // avalon bridge
  wire        w_req;
  wire        w_wr;
  wire [7:0]  w_adrs;
  wire        w_ack;
  wire        w_be;
  wire [7:0]  w_wd;
  wire        w_rstr;
  wire [7:0]  w_rd;
  // USB host
  wire           w_usb_ack;
  wire           w_host_sof_int;
  wire           w_host_connect_int;
  wire           w_host_resume_int;
  wire           w_host_trans_int;
  wire  [1:0]    w_host_usb_wire_data_in;
  wire  [1:0]    w_host_usb_wire_data_out;
  wire           w_host_usb_wire_ctrl;
  wire           w_usb_fullspeed;
  assign o_int = 
                 w_host_sof_int|
                 w_host_connect_int|
                 w_host_resume_int|
                 w_host_trans_int;
  assign usb_dp = (w_host_usb_wire_ctrl) ? w_host_usb_wire_data_out[1] : 1'hz;
  assign usb_dn = (w_host_usb_wire_ctrl) ? w_host_usb_wire_data_out[0] : 1'hz;
  assign w_host_usb_wire_data_in[0] = usb_dn;
  assign w_host_usb_wire_data_in[1] = usb_dp;

  assign w_rstr = (w_req & !w_wr & w_ack);
   
fm_avalon #(.P_AVALON_ADR_WIDTH(6),
            .P_INTERNAL_ADR_WIDTH(8),
            .P_INTERNAL_BE_WIDTH(1),
            .P_INTERNAL_DATA_WIDTH(8)
           ) u_avalon (
  .clk_core(clk_core),
  .rst_x(rst_x),
  // AVALON bus
  .i_av_adr(i_av_adr),
  .i_av_be(i_av_be),
  .i_av_r(i_av_r),
  .o_av_rd(o_av_rd),
  .i_av_w(i_av_w),
  .i_av_wd(i_av_wd),
  .o_av_wait(o_av_wait),
  // internal side
  .o_req(w_req),
  .o_wr(w_wr),
  .o_adrs(w_adrs),
  .i_ack(w_ack),
  .o_be(w_be),
  .o_wd(w_wd),
  .i_rstr(w_rstr),
  .i_rd(w_rd)
);

usbHost u_usbHost (
  .clk_i(clk_core),
  .rst_i(~rst_x),
  .address_i(w_adrs),
  .data_i(w_wd),
  .data_o(w_rd),
  .we_i(w_wr),
  .strobe_i(w_req),
  .ack_o(w_ack),
  .usbClk(clk48m),
  .hostSOFSentIntOut(w_host_sof_int),
  .hostConnEventIntOut(w_host_connect_int),
  .hostResumeIntOut(w_host_resume_int),
  .hostTransDoneIntOut(w_host_trans_int),
  .USBWireDataIn(w_host_usb_wire_data_in),
  .USBWireDataInTick(),
  .USBWireDataOut(w_host_usb_wire_data_out),
  .USBWireDataOutTick(),
  .USBWireCtrlOut(w_host_usb_wire_ctrl),
  .USBFullSpeed(w_usb_fullspeed)
);

endmodule
